import 'package:grizzly/src/utils/comap.dart';

extension BoolIterableExt on Iterable<bool?> {
  bool get isTrue {
    for (final item in this) {
      if (item == null) continue;
      if (!item) return false;
    }
    return true;
  }

  bool get isFalse {
    for (final item in this) {
      if (item == null) continue;
      if (item) return false;
    }
    return true;
  }

  int get sum {
    int sum = 0;
    for (final item in this) {
      if (item == null) continue;
      if (item) sum++;
    }
    return sum;
  }

  double get mean {
    if (length == 0) return 0.0;
    return sum / length;
  }

  Iterable<int?> toInts({int trueValue = 1, int falseValue = 0}) => map((item) {
        if (item == null) {
          return null;
        } else {
          return item ? trueValue : falseValue;
        }
      });

  Iterable<String?> toStrings(
          {String trueVal = 'True', String falseVal = 'False'}) =>
      map((e) {
        if (e == null) {
          return null;
        } else {
          return e ? trueVal : falseVal;
        }
      });

  Iterable<bool?> operator ~() => map((e) {
        if (e == null) {
          return null;
        } else {
          return !e;
        }
      });

  Iterable<bool?> operator |(Iterable<bool?> other) =>
      comap<bool?, bool?, bool?>(this, other, (a, b) {
        if (a == null || b == null) {
          return null;
        }
        return a || b;
      });

  Iterable<bool?> operator &(Iterable<bool?> other) =>
      comap<bool?, bool?, bool?>(this, other, (a, b) {
        if (a == null || b == null) {
          return null;
        }
        return a && b;
      });
}
