import 'package:grizzly/src/utils/norepeat_sampler.dart';

extension ComparableList<T extends Comparable> on List<T> {
  void zort({bool descending = false}) {
    if (isEmpty) return;

    if (!descending) {
      sort();
    } else {
      sort((T a, T b) => b.compareTo(a));
    }
  }

  List<bool> ge(other) {
    final ret = List<bool>.filled(length, false, growable: true);
    if (other is T) {
      for (int i = 0; i < length; i++) {
        ret[i] = this[i].compareTo(other) >= 0;
      }
    } else if (other is Iterable<T>) {
      if (other.length != length) {
        throw LengthMismatch(
            expected: length, found: other.length, subject: 'other');
      }

      int i = 0;
      for (final otherItem in other) {
        ret[i] = this[i].compareTo(otherItem) >= 0;
        i++;
      }
    } else {
      throw UnsupportedError('Type not supported!');
    }
    return ret;
  }

  List<bool> gt(other) {
    final ret = List<bool>.filled(length, false, growable: true);
    if (other is T) {
      for (int i = 0; i < length; i++) {
        ret[i] = this[i].compareTo(other) > 0;
      }
    } else if (other is Iterable<T>) {
      if (other.length != length) {
        throw LengthMismatch(
            expected: length, found: other.length, subject: 'other');
      }

      int i = 0;
      for (final otherItem in other) {
        ret[i] = this[i].compareTo(otherItem) > 0;
        i++;
      }
    } else {
      throw UnsupportedError('Type not supported!');
    }
    return ret;
  }

  List<bool> le(other) {
    final ret = List<bool>.filled(length, false, growable: true);
    if (other is T) {
      for (int i = 0; i < length; i++) {
        ret[i] = this[i].compareTo(other) <= 0;
      }
    } else if (other is Iterable<T>) {
      if (other.length != length) {
        throw LengthMismatch(
            expected: length, found: other.length, subject: 'other');
      }

      int i = 0;
      for (final otherItem in other) {
        ret[i] = this[i].compareTo(otherItem) <= 0;
        i++;
      }
    } else {
      throw UnsupportedError('Type not supported!');
    }
    return ret;
  }

  List<bool> lt(other) {
    final ret = List<bool>.filled(length, false, growable: true);
    if (other is T) {
      for (int i = 0; i < length; i++) {
        ret[i] = this[i].compareTo(other) < 0;
      }
    } else if (other is Iterable<T>) {
      if (other.length != length) {
        throw LengthMismatch(
            expected: length, found: other.length, subject: 'other');
      }

      int i = 0;
      for (final otherItem in other) {
        ret[i] = this[i].compareTo(otherItem) < 0;
        i++;
      }
    } else {
      throw UnsupportedError('Type not supported!');
    }
    return ret;
  }
}
