import 'package:collection/collection.dart';
import 'package:grizzly/grizzly.dart';

typedef GroupingFunction<T> = T Function(Map<String, dynamic> row);

class Grouper<T> {
  final String name;

  final GroupingFunction<T> grouper;

  Grouper(this.name, this.grouper);
}

extension GroupingExpressionsExt<T> on List<Grouper<T>> {
  DataMap<String, T> makeGroupKey(Map<String, dynamic> row) {
    final group = DataMap<String, T>({});
    for (final grouper in this) {
      group[grouper.name] = grouper.grouper(row);
    }
    return group;
  }
}

Grouper byCol(String column, {String? name}) =>
    Grouper(name ?? column, (row) => row[column]);

class Grouped<T> {
  final Named2DView<T> _inner;

  // TODO final Iterable<DataRow> _iterable;

  final List<Grouper> _groupers;

  Grouped(this._inner, this._groupers) {
    // TODO optimize with frame version
  }

  Set<DataMap> get groupKeys {
    final ret = <DataMap>{};
    for (final row in _inner.rowMaps) {
      final group = _groupers.makeGroupKey(row);
      ret.add(group);
    }
    return ret;
  }

  Map<DataMap<String, dynamic>, int> get counts =>
      _inner.rowMaps.map(_groupers.makeGroupKey).valueCounts();

  Map<DataMap<String, dynamic>, Named2D> materialize() {
    final ret = <DataMap<String, dynamic>, Named2D>{};

    for (final row in _inner.rowMaps) {
      final groupKey = _groupers.makeGroupKey(row);
      ret[groupKey] = (ret[groupKey] ?? Named2D([], names: _inner.names))
        ..addMap(row);
    }

    return ret;
  }

  /// Materializes a specific group
  Named2D<E> materializeGroup<E>(Map<String, dynamic> group) {
    group = DataMap(group);
    final ret = Named2D<E>([], names: _inner.names);

    for (final row in _inner.rowMaps) {
      final groupKey = _groupers.makeGroupKey(row);
      if (groupKey != group) {
        continue;
      }
      ret.addMap(row);
    }

    return ret;
  }

  Named2D aggregate(Aggregator aggregator) =>
      materialize().entries.map((e) => aggregator(e.key, e.value)).named();
}

typedef Aggregator<E> = Map<String, E> Function(
    DataMap<String, dynamic> keys, Named2D rows);

class DataMap<K, T> extends DelegatingMap<K, T> {
  DataMap(Map<K, T> inner) : super(inner);

  @override
  bool operator ==(other) {
    if (other is! Map<String, T>) {
      return false;
    }

    if (length != other.length) {
      return false;
    }

    for (final key in keys) {
      if (this[key] != other[key]) {
        return false;
      }
    }

    return true;
  }

  @override
  int get hashCode => Object.hashAll(values);
}
