import 'package:meta/meta.dart';
import 'dart:collection';

@experimental
class IndexedList<E> extends ListBase<E> {
  final List<E> _inner = [];

  final _indexMap = <E, SplayTreeSet<int>>{};

  IndexedList(Iterable<E> data) {
    addAll(data);
  }

  @override
  E operator [](int index) => _inner[index];

  @override
  void operator []=(int index, E value) {
    final old = this[index];
    _inner[index] = value;
    _removeIndex(old, index);
    _addIndex(value, index);
  }

  @override
  int get length => _inner.length;

  @override
  set length(int value) {
    if (value > length) {
      for (int i = length; i < value; i++) {
        _addIndex(null, i);
      }
    } else {
      for (int i = value; i < length; i++) {
        _removeIndex(this[i], i);
      }
    }
    _inner.length = value;
  }

  @override
  void add(E element) {
    _inner.add(element);
    _addIndex(element, _inner.length - 1);
  }

  @override
  bool contains(Object? element) => _indexMap.containsKey(element);

  @override
  int indexOf(Object? element, [int start = 0]) {
    final indices = _indexMap[element];
    if (indices == null) return -1;
    return indices.firstWhere((value) => value >= start, orElse: () => -1);
  }

  List<int> indicesOf(Object? element, [int start = 0]) {
    final indices = _indexMap[element];
    if (indices == null) return [];
    return indices.where((value) => value >= start).toList();
  }

  void _addIndex(E? value, int index) {
    _indexMap[value as E] = (_indexMap[value] ?? SplayTreeSet((a, b) => a - b))
      ..add(index);
  }

  void _removeIndex(E value, int index) {
    final indices = _indexMap[value];
    if (indices == null) {
      throw Exception('corrupted state');
    }
    if (!indices.remove(index)) {
      throw Exception('corrupted state');
    }
    if (indices.isEmpty) {
      _indexMap.remove(value);
    }
  }

  void _remakeIndices() {
    _indexMap.clear();
    int i = 0;
    for (final item in this) {
      _addIndex(item, i);
      i++;
    }
  }
}
