import 'dart:math';

typedef Double1D = List<double>;

extension DoubleListExt on Double1D {
  void negate() {
    for (int i = 0; i < length; i++) {
      this[i] = -this[i];
    }
  }

  void assignAddition(/* Iterable<num> | num */ other) {
    if (other is Iterable<num>) {
      final iter = other.iterator;
      for (int i = 0; i < length; i++) {
        if (!iter.moveNext()) {
          return;
        }
        this[i] = this[i] + iter.current;
      }
    } else if (other is num) {
      for (int i = 0; i < length; i++) {
        this[i] = this[i] + other;
      }
    } else {
      throw ArgumentError('Type not supported', 'other');
    }
  }

  void assignSubtraction(/* Iterable<num> | num */ other) {
    if (other is Iterable<num>) {
      final iter = other.iterator;
      for (int i = 0; i < length; i++) {
        if (!iter.moveNext()) {
          return;
        }
        this[i] = this[i] - iter.current;
      }
    } else if (other is num) {
      for (int i = 0; i < length; i++) {
        this[i] = this[i] - other;
      }
    } else {
      throw ArgumentError('Type not supported', 'other');
    }
  }

  void assignMultiplication(/* Iterable<num> | num */ other) {
    if (other is Iterable<num>) {
      final iter = other.iterator;
      for (int i = 0; i < length; i++) {
        if (!iter.moveNext()) {
          return;
        }
        this[i] = this[i] * iter.current;
      }
    } else if (other is num) {
      for (int i = 0; i < length; i++) {
        this[i] = this[i] * other;
      }
    } else {
      throw ArgumentError('Type not supported', 'other');
    }
  }

  void assignDivision(/* Iterable<num> | num */ other) {
    if (other is Iterable<num>) {
      final iter = other.iterator;
      for (int i = 0; i < length; i++) {
        if (!iter.moveNext()) {
          return;
        }
        this[i] = this[i] / iter.current;
      }
    } else if (other is num) {
      for (int i = 0; i < length; i++) {
        this[i] = this[i] / other;
      }
    } else {
      throw ArgumentError('Type not supported', 'other');
    }
  }

  void assignTruncDivision(/* Iterable<num> | num */ other) {
    if (other is Iterable<num>) {
      final iter = other.iterator;
      for (int i = 0; i < length; i++) {
        if (!iter.moveNext()) {
          return;
        }
        this[i] = (this[i] ~/ iter.current).toDouble();
      }
    } else if (other is num) {
      for (int i = 0; i < length; i++) {
        this[i] = (this[i] ~/ other).toDouble();
      }
    } else {
      throw ArgumentError('Type not supported', 'other');
    }
  }

  void assignLog() {
    for (int i = 0; i < length; i++) {
      this[i] = log(this[i]);
    }
  }

  void assignLog10() {
    for (int i = 0; i < length; i++) {
      this[i] = log(this[i]) / ln10;
    }
  }

  void assignExp() {
    for (int i = 0; i < length; i++) {
      this[i] = exp(this[i]);
    }
  }

  void assignLogN(double n) {
    for (int i = 0; i < length; i++) {
      this[i] = log(this[i]) / log(n);
    }
  }

  void assignSqrt(double n) {
    for (int i = 0; i < length; i++) {
      this[i] = sqrt(this[i]);
    }
  }

  void assignFloor() {
    for (int i = 0; i < length; i++) {
      this[i] = this[i].floorToDouble();
    }
  }

  void assignCeil() {
    for (int i = 0; i < length; i++) {
      this[i] = this[i].ceilToDouble();
    }
  }

  void clip({double? min, double? max}) {
    if (min != null && max != null) {
      for (int i = 0; i < length; i++) {
        final double d = this[i];

        if (d < min) this[i] = min;
        if (d > max) this[i] = max;
      }
      return;
    }
    if (min != null) {
      for (int i = 0; i < length; i++) {
        final double d = this[i];

        if (d < min) this[i] = min;
      }
      return;
    }
    if (max != null) {
      for (int i = 0; i < length; i++) {
        final double d = this[i];

        if (d > max) this[i] = max;
      }
      return;
    }
  }
}
