import 'package:grizzly/grizzly.dart';

typedef Double2D = List<List<double>>;

extension Double on double {
  static List<List<double>> identity(int size, {double fill = 1.0}) =>
      List.generate(size, (i) => List.generate(size, (j) => i == j ? fill : 0));

  static List<List<double>> sized(int rows, int cols, {double fill = 0}) =>
      List<List<double>>.generate(
          rows, (index) => List<double>.filled(cols, fill));
}

extension DoubleMatExt on Double2D {
  void assignAddition(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num || other is Iterable<num>) {
      for (final row in this) {
        row.assignAddition(other);
      }
    } else if (other is Iterable<Iterable<num>>) {
      final iter = other.iterator;
      for (final row in this) {
        if (!iter.moveNext()) {
          break;
        }
        row.assignAddition(iter.current);
      }
    } else {
      throw UnsupportedError(
          'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
    }
  }

  void assignSubtraction(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num || other is Iterable<num>) {
      for (final row in this) {
        row.assignSubtraction(other);
      }
    } else if (other is Iterable<Iterable<num>>) {
      final iter = other.iterator;
      for (final row in this) {
        if (!iter.moveNext()) {
          break;
        }
        row.assignSubtraction(iter.current);
      }
    } else {
      throw UnsupportedError(
          'only Iterable<Iterable<num>> | Iterable<num> | num is supported. got type ${other?.runtimeType}');
    }
  }

  void assignMultiplication(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num || other is Iterable<num>) {
      for (final row in this) {
        row.assignMultiplication(other);
      }
    } else if (other is Iterable<Iterable<num>>) {
      final iter = other.iterator;
      for (final row in this) {
        if (!iter.moveNext()) {
          break;
        }
        row.assignMultiplication(iter.current);
      }
    } else {
      throw UnsupportedError(
          'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
    }
  }

  void assignDivision(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num || other is Iterable<num>) {
      for (final row in this) {
        row.assignDivision(other);
      }
    } else if (other is Iterable<Iterable<num>>) {
      final iter = other.iterator;
      for (final row in this) {
        if (!iter.moveNext()) {
          break;
        }
        row.assignDivision(iter.current);
      }
    } else {
      throw UnsupportedError(
          'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
    }
  }

  void assignTruncDivision(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num || other is Iterable<num>) {
      for (final row in this) {
        row.assignTruncDivision(other);
      }
    } else if (other is Iterable<Iterable<num>>) {
      final iter = other.iterator;
      for (final row in this) {
        if (!iter.moveNext()) {
          break;
        }
        row.assignTruncDivision(iter.current);
      }
    } else {
      throw UnsupportedError(
          'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
    }
  }

  Double2D matmul(List<List<num>> other) {
    if (numCols != other.numRows) throw Exception('Invalid size!');

    Double2D ret = Double.sized(numRows, other.numCols);

    for (int i = 0; i < numRows; i++) {
      for (int j = 0; j < other.numCols; j++) {
        double v = 0.0;
        for (int ri = 0; ri < numCols; ri++) {
          v += this[i][ri] * other[ri][j];
        }
        ret[i][j] = v;
      }
    }
    return ret;
  }

  Double2D matmulDiag(Iterable<num> other) {
    if (numCols != other.length) throw Exception('Invalid size!');

    Double2D ret = Double.sized(numRows, other.length);

    for (int i = 0; i < numRows; i++) {
      int j = 0;
      for (final v in other) {
        ret[i][j] = this[i][j] * v;
        j++;
      }
    }
    return ret;
  }
}
