import 'dart:collection';
import 'package:grizzly/grizzly.dart';

class ColsView<T> extends IterableBase<Iterable<T>> {
  final Iterable<Iterable<T>> _inner;

  ColsView(this._inner);

  @override
  Iterator<Iterable<T>> get iterator =>
      indices(_inner.numCols).map((c) => _inner.map((row) => row[c])).iterator;
}

class Cols<T> extends ListBase<List<T>> implements List<List<T>> {
  final List<List<T>> _inner;

  Cols(this._inner);

  @override
  List<T> operator [](/* int | String */ dynamic index) {
    if (index is String) {
      if (_inner is Named2D) {
        final intIndex = (_inner as Named2D).loc(index);
        if (intIndex < 0) {
          throw ArgumentError.value(index, 'index', 'not found');
        }
        return NamedList(_inner.map((row) => row[intIndex]).toList(),
            name: index);
      } else {
        throw ArgumentError.value(index, 'index', 'should be int or String');
      }
    }
    return _inner.map((e) => e[index]).toList();
  }

  @override
  void operator []=(/* int | String */ dynamic index, List<T> value) {
    if (index is String) {
      if (_inner is Named2D) {
        index = (_inner as Named2D).loc(index);
      } else {
        throw ArgumentError.value(index, 'index', 'should be int or String');
      }
    }

    final thisIter = _inner.iterator;
    final valIter = value.iterator;
    while (thisIter.moveNext() && valIter.moveNext()) {
      thisIter.current[index] = valIter.current;
    }
  }

  void addScalar(T v) {
    for (final row in _inner) {
      row.add(v);
    }
  }

  // TODO col view

  @override
  int get length {
    if (_inner.isEmpty) return 0;
    return _inner.first.length;
  }

  @override
  set length(int value) {
    for (final item in this) {
      item.length = value;
    }
  }

  int get numRows => _inner.numCols;

  int get numCols => _inner.numRows;
}
