import 'package:grizzly/src/utils/comap.dart';
import 'package:grizzly/grizzly.dart';
import 'package:grizzly/src/utils/reducers.dart';

typedef Int1D = List<int>;

extension IntNullIterableExt on Iterable<int?> {
  int get sum {
    int ret = 0;
    for (final item in this) {
      if (item == null) continue;
      ret += item;
    }
    return ret;
  }

  int get prod {
    int ret = 1;
    for (final item in this) {
      if (item == null) continue;
      ret *= item;
    }
    return ret;
  }

  double get mean {
    if (length == 0) return 0.0;

    final reducer = MeanReducer();
    for (var item in this) {
      reducer.consume(item ?? 0);
    }
    return reducer.mean;
  }

  double get variance {
    if (length == 0) return 0.0;

    final reducer = VarianceReducer();
    for (var item in this) {
      reducer.consume(item ?? 0);
    }
    return reducer.variance;
  }

  int dot([Iterable<num?>? other]) => cofold<int, int?, num?>(
      0,
      this,
      other ?? this,
      (value, int? a, num? b) => value + (a ?? 0) * (b?.toInt() ?? 0));

  Iterable<int> matmult(List<List<int>> matrix) =>
      matrix.cols.map((col) => dot(col));
}

extension IntIterableExt on Iterable<int> {
  bool isEqual(Iterable<num> v) {
    if (length != v.length) return false;
    final iter1 = iterator;
    final iter2 = v.iterator;
    while (iter1.moveNext() && iter2.moveNext()) {
      if (iter1.current != iter2.current) return false;
    }
    return true;
  }

  void assertEqual(Iterable<num> v) {
    if (length != v.length) {
      throw AssertionError('Length mismatch');
    }
    final iter1 = iterator;
    final iter2 = v.iterator;
    int i = 0;
    while (iter1.moveNext() && iter2.moveNext()) {
      if (iter1.current != iter2.current) {
        throw AssertionError('$i =>  ${iter1.current} != ${iter2.current}');
      }
      i++;
    }
  }

  bool isEqualScalar(num v) {
    for (final item in this) {
      if (item != v) return false;
    }
    return true;
  }
}
