import 'dart:math' as math;
import 'dart:collection';

final math.Random _rand = math.Random();

List<T> noRepeatSampler<T>(List<T> population, int count,
    {math.Random? random}) {
  random ??= _rand;
  final int n = population.length;

  if (count > n) {
    return population.toList()..shuffle(random);
  }

  final samples = List<T?>.filled(count, null);

  if (n < 1000) {
    final unpicked = population.toList();
    for (int i = 0; i < count; i++) {
      final sampleIdx = random.nextInt(n - i);
      samples[i] = unpicked[sampleIdx];
      unpicked[sampleIdx] = unpicked[n - i - 1];
    }
  } else {
    final picked = SplayTreeSet<int>();
    for (int i = 0; i < count; i++) {
      final int sampleIdx = () {
        int newIdx;
        do {
          newIdx = random!.nextInt(n);
        } while (picked.contains(newIdx));
        return newIdx;
      }();
      picked.add(sampleIdx);
      samples[i] = population[sampleIdx];
    }
  }

  return samples.cast();
}

class LengthMismatch implements Exception {
  final int? expected;

  final int? found;

  final String? subject;

  const LengthMismatch({this.expected, this.found, this.subject});

  @override
  String toString() {
    final sb = StringBuffer();

    sb.write('Length mismatch');

    if (subject != null) sb.write(' for $subject');

    if (expected != null) {
      sb.write('! Expected $expected');

      if (found != null) {
        sb.write(' found $found');
      }
    }

    sb.write('!');
    return sb.toString();
  }
}
